@@ -20,17 +20,13 @@ try:
 except NameError:
     from sets import Set as set
 
-import sys
-import warnings
-
-from py23compat import rsplit
-
 import sqlalchemy
 from sqlalchemy.types import *
 
 from elixir.options import using_options, using_table_options, \
-                           using_mapper_options, options_defaults
-from elixir.entity import Entity, EntityMeta, EntityDescriptor, \
+                           using_mapper_options, options_defaults, \
+                           using_options_defaults
+from elixir.entity import Entity, EntityBase, EntityMeta, EntityDescriptor, \
                           setup_entities, cleanup_entities
 from elixir.fields import has_field, Field
 from elixir.relationships import belongs_to, has_one, has_many, \
@@ -39,17 +35,20 @@ from elixir.relationships import belongs_to, has_one, has_many, \
 from elixir.properties import has_property, GenericProperty, ColumnProperty, \
                               Synonym
 from elixir.statements import Statement
+from elixir.collection import EntityCollection, GlobalEntityCollection
 
 
-__version__ = '0.6.1'
+__version__ = '0.7.1'
 
-__all__ = ['Entity', 'EntityMeta', 'EntityCollection', 'entities',
+__all__ = ['Entity', 'EntityBase', 'EntityMeta', 'EntityCollection',
+           'entities',
            'Field', 'has_field',
            'has_property', 'GenericProperty', 'ColumnProperty', 'Synonym',
            'belongs_to', 'has_one', 'has_many', 'has_and_belongs_to_many',
            'ManyToOne', 'OneToOne', 'OneToMany', 'ManyToMany',
            'using_options', 'using_table_options', 'using_mapper_options',
-           'options_defaults', 'metadata', 'session',
+           'options_defaults', 'using_options_defaults',
+           'metadata', 'session',
            'create_all', 'drop_all',
            'setup_all', 'cleanup_all',
            'setup_entities', 'cleanup_entities'] + \
@@ -68,57 +67,7 @@ metadata = sqlalchemy.MetaData()
 metadatas = set()
 
 # default entity collection
-class EntityCollection(list):
-    def __init__(self):
-        # _entities is a dict of entities keyed on their name.
-        self._entities = {}
-        list.__init__(self)
-
-    def append(self, entity):
-        '''
-        Add an entity to the collection.
-        '''
-        super(EntityCollection, self).append(entity)
-
-        key = entity.__name__
-        mapped_entity = self._entities.get(key)
-        if mapped_entity:
-            if isinstance(mapped_entity, list):
-                mapped_entity.append(entity)
-            else:
-                self._entities[key] = [mapped_entity, entity]
-        else:
-            self._entities[key] = entity
-
-    def resolve(self, key, entity=None):
-        '''
-        Resolve a key to an Entity. The optional `entity` argument is the
-        "source" entity when resolving relationship targets.
-        '''
-        path = rsplit(key, '.', 1)
-        classname = path.pop()
-        if path:
-            # Do we have a fully qualified entity name?
-            module = sys.modules[path.pop()]
-            return getattr(module, classname, None)
-        else:
-            # Otherwise we look in the entities of this collection
-            res = self._entities[key]
-            if isinstance(res, list):
-                raise Exception("'%s' resolves to several entities, you should "
-                                "use the full path (including the full module "
-                                "name) to that entity." % key)
-            else:
-                return res
-
-    def clear(self):
-        self._entities = {}
-        del self[:]
-
-    def __getattr__(self, key):
-        return self.resolve(key)
-
-entities = EntityCollection()
+entities = GlobalEntityCollection()
 
 
 def create_all(*args, **kwargs):
