@@ -3,97 +3,140 @@ This module provides the ``Entity`` base class, as well as its metaclass
 ``EntityMeta``.
 '''
 
-from py23compat import set, rsplit, sorted
+from py23compat import sorted
 
 import sys
-import inspect
+import types
 import warnings
-from copy import copy
+
+from copy import deepcopy
 
 import sqlalchemy
-from sqlalchemy     import Table, Column, Integer, desc, ForeignKey, and_, \
-                           ForeignKeyConstraint
-from sqlalchemy.orm import Query, MapperExtension, mapper, object_session, \
+from sqlalchemy import Table, Column, Integer, desc, ForeignKey, and_, \
+                       ForeignKeyConstraint
+from sqlalchemy.orm import MapperExtension, mapper, object_session, \
                            EXT_CONTINUE, polymorphic_union, ScopedSession, \
                            ColumnProperty
+from sqlalchemy.sql import ColumnCollection
 
 import elixir
-from elixir.statements import process_mutators
+from elixir.statements import process_mutators, MUTATORS
 from elixir import options
 from elixir.properties import Property
 
 DEBUG = False
+try:
+    from sqlalchemy.orm import EXT_PASS
+    SA05orlater = False
+except ImportError:
+    SA05orlater = True
 
 __doc_all__ = ['Entity', 'EntityMeta']
 
 
+def session_mapper_factory(scoped_session):
+    def session_mapper(cls, *args, **kwargs):
+        if kwargs.pop('save_on_init', True):
+            old_init = cls.__init__
+            def __init__(self, *args, **kwargs):
+                old_init(self, *args, **kwargs)
+                scoped_session.add(self)
+            cls.__init__ = __init__
+        cls.query = scoped_session.query_property()
+        return mapper(cls, *args, **kwargs)
+    return session_mapper
+
+
 class EntityDescriptor(object):
     '''
     EntityDescriptor describes fields and options needed for table creation.
     '''
 
     def __init__(self, entity):
-        entity.table = None
-        entity.mapper = None
-
         self.entity = entity
-        self.module = sys.modules[entity.__module__]
+        self.parent = None
 
-        self.has_pk = False
-        self._pk_col_done = False
+        bases = []
+        for base in entity.__bases__:
+            if isinstance(base, EntityMeta):
+                if is_entity(base) and not is_abstract_entity(base):
+                    if self.parent:
+                        raise Exception(
+                            '%s entity inherits from several entities, '
+                            'and this is not supported.'
+                            % self.entity.__name__)
+                    else:
+                        self.parent = base
+                        bases.extend(base._descriptor.bases)
+                        self.parent._descriptor.children.append(entity)
+                else:
+                    bases.append(base)
+        self.bases = bases
+        if not is_entity(entity) or is_abstract_entity(entity):
+            return
+
+        # entity.__module__ is not always reliable (eg in mod_python)
+        self.module = sys.modules.get(entity.__module__)
 
         self.builders = []
 
-        self.parent = None
         #XXX: use entity.__subclasses__ ?
         self.children = []
 
-        for base in entity.__bases__:
-            if isinstance(base, EntityMeta) and is_entity(base):
-                if self.parent:
-                    raise Exception('%s entity inherits from several entities,'
-                                    ' and this is not supported.'
-                                    % self.entity.__name__)
-                else:
-                    self.parent = base
-                    self.parent._descriptor.children.append(entity)
+        # used for multi-table inheritance
+        self.join_condition = None
+        self.has_pk = False
+        self._pk_col_done = False
 
         # columns and constraints waiting for a table to exist
-        self._columns = list()
-        self.constraints = list()
+        self._columns = ColumnCollection()
+        self.constraints = []
 
         # properties (it is only useful for checking dupe properties at the
         # moment, and when adding properties before the mapper is created,
         # which shouldn't happen).
-        self.properties = dict()
+        self.properties = {}
 
         #
-        self.relationships = list()
+        self.relationships = []
 
         # set default value for options
-        self.order_by = None
-        self.table_args = list()
+        self.table_args = []
 
-        # set default value for options with an optional module-level default
-        self.metadata = getattr(self.module, '__metadata__', elixir.metadata)
-        self.session = getattr(self.module, '__session__', elixir.session)
-        self.collection = getattr(self.module, '__entity_collection__',
-                                  elixir.entities)
+        # base class(es) options_defaults
+        options_defaults = self.options_defaults()
 
-        for option in ('autosetup', 'inheritance', 'polymorphic', 'identity',
-                       'autoload', 'tablename', 'shortnames',
-                       'auto_primarykey', 'version_id_col',
-                       'allowcoloverride'):
-            setattr(self, option, options.options_defaults[option])
+        complete_defaults = options.options_defaults.copy()
+        complete_defaults.update({
+            'metadata': elixir.metadata,
+            'session': elixir.session,
+            'collection': elixir.entities
+        })
 
-        for option_dict in ('mapper_options', 'table_options'):
-            setattr(self, option_dict,
-                    options.options_defaults[option_dict].copy())
+        # set default value for other options
+        for key in options.valid_options:
+            value = options_defaults.get(key, complete_defaults[key])
+            if isinstance(value, dict):
+                value = value.copy()
+            setattr(self, key, value)
+
+        # override options with module-level defaults defined
+        for key in ('metadata', 'session', 'collection'):
+            attr = '__%s__' % key
+            if hasattr(self.module, attr):
+                setattr(self, key, getattr(self.module, attr))
+
+    def options_defaults(self):
+        base_defaults = {}
+        for base in self.bases:
+            base_defaults.update(base._descriptor.options_defaults())
+        base_defaults.update(getattr(self.entity, 'options_defaults', {}))
+        return base_defaults
 
     def setup_options(self):
         '''
-        Setup any values that might depend on using_options. For example, the
-        tablename or the metadata.
+        Setup any values that might depend on the "using_options" class
+        mutator. For example, the tablename or the metadata.
         '''
         elixir.metadatas.add(self.metadata)
         if self.collection is not None:
@@ -111,20 +154,21 @@ class EntityDescriptor(object):
                 modulename = entity.__module__.replace('.', '_')
                 tablename = "%s_%s" % (modulename, entity.__name__)
                 self.tablename = tablename.lower()
-        elif callable(self.tablename):
+        elif hasattr(self.tablename, '__call__'):
             self.tablename = self.tablename(entity)
 
         if not self.identity:
             if 'polymorphic_identity' in self.mapper_options:
                 self.identity = self.mapper_options['polymorphic_identity']
             else:
-                #TODO: include module name
+                #TODO: include module name (We could have b.Account inherit
+                # from a.Account)
                 self.identity = entity.__name__.lower()
-        elif 'polymorphic_identity' in kwargs:
+        elif 'polymorphic_identity' in self.mapper_options:
             raise Exception('You cannot use the "identity" option and the '
                             'polymorphic_identity mapper option at the same '
                             'time.')
-        elif callable(self.identity):
+        elif hasattr(self.identity, '__call__'):
             self.identity = self.identity(entity)
 
         if self.polymorphic:
@@ -160,6 +204,7 @@ class EntityDescriptor(object):
                     # key columns
                     parent_desc = self.parent._descriptor
                     tablename = parent_desc.table_fullname
+                    join_clauses = []
                     for pk_col in parent_desc.primary_keys:
                         colname = options.MULTIINHERITANCECOL_NAMEFORMAT % \
                                   {'entity': self.parent.__name__.lower(),
@@ -173,6 +218,8 @@ class EntityDescriptor(object):
                         col = Column(colname, pk_col.type, fk,
                                      primary_key=True)
                         self.add_column(col)
+                        join_clauses.append(col == pk_col)
+                    self.join_condition = and_(*join_clauses)
                 elif self.inheritance == 'concrete':
                     # Copy primary key columns from the parent.
                     for col in self.parent._descriptor.columns:
@@ -200,7 +247,7 @@ class EntityDescriptor(object):
         Create a SQLAlchemy table-object with all columns that have been
         defined up to this point.
         '''
-        if self.entity.table:
+        if self.entity.table is not None:
             return
 
         if self.autoload != only_autoloaded:
@@ -219,7 +266,7 @@ class EntityDescriptor(object):
                     # re-add the entity columns to the parent entity so that
                     # they are added to the parent's table (whether the
                     # parent's table is already setup or not).
-                    for col in self.columns:
+                    for col in self._columns:
                         self.parent._descriptor.add_column(col)
                     for constraint in self.constraints:
                         self.parent._descriptor.add_constraint(constraint)
@@ -235,12 +282,10 @@ class EntityDescriptor(object):
                         if not col.primary_key:
                             self.add_column(col.copy())
 
-                    #FIXME: use the public equivalent of _get_colspec when
-                    #available
                     for con in self.parent._descriptor.constraints:
                         self.add_constraint(
                             ForeignKeyConstraint(
-                                [c.key for c in con.columns],
+                                [e.parent.key for e in con.elements],
                                 [e._get_colspec() for e in con.elements],
                                 name=con.name, #TODO: modify it
                                 onupdate=con.onupdate, ondelete=con.ondelete,
@@ -257,10 +302,11 @@ class EntityDescriptor(object):
                     self.version_id_col = options.DEFAULT_VERSION_ID_COL_NAME
                 self.add_column(Column(self.version_id_col, Integer))
 
-            args = self.columns + self.constraints + self.table_args
-
+            args = list(self.columns) + self.constraints + self.table_args
         self.entity.table = Table(self.tablename, self.metadata,
                                   *args, **kwargs)
+        if DEBUG:
+            print self.entity.table.repr2()
 
     def setup_reltables(self):
         self.call_builders('create_tables')
@@ -287,11 +333,15 @@ class EntityDescriptor(object):
 
         # create a list of callbacks for each event
         methods = {}
-        for name, method in self.entity.__dict__.items():
-            if hasattr(method, '_elixir_events'):
-                for event in method._elixir_events:
-                    event_methods = methods.setdefault(event, [])
-                    event_methods.append(method)
+
+        all_methods = getmembers(self.entity,
+                                 lambda a: isinstance(a, types.MethodType))
+
+        for name, method in all_methods:
+            for event in getattr(method, '_elixir_events', []):
+                event_methods = methods.setdefault(event, [])
+                event_methods.append(method)
+
         if not methods:
             return
 
@@ -318,7 +368,7 @@ class EntityDescriptor(object):
         if isinstance(order_by, basestring):
             order_by = [order_by]
 
-        order = list()
+        order = []
         for colname in order_by:
             col = self.get_column(colname.strip('-'))
             if colname.startswith('-'):
@@ -339,7 +389,7 @@ class EntityDescriptor(object):
         # inheritance scenario as demonstrated in
         # sqlalchemy/test/orm/inheritance/concrete.py
         # this should be added along other
-        kwargs = self.mapper_options
+        kwargs = {}
         if self.order_by:
             kwargs['order_by'] = self.translate_order_by(self.order_by)
 
@@ -348,14 +398,12 @@ class EntityDescriptor(object):
 
         if self.inheritance in ('single', 'concrete', 'multi'):
             if self.parent and \
-               not (self.inheritance == 'concrete' and not self.polymorphic):
+               (self.inheritance != 'concrete' or self.polymorphic):
+                # non-polymorphic concrete doesn't need this
                 kwargs['inherits'] = self.parent.mapper
 
             if self.inheritance == 'multi' and self.parent:
-                col_pairs = zip(self.primary_keys,
-                                self.parent._descriptor.primary_keys)
-                kwargs['inherit_condition'] = \
-                    and_(*[pc == c for c, pc in col_pairs])
+                kwargs['inherit_condition'] = self.join_condition
 
             if self.polymorphic:
                 if self.children:
@@ -363,11 +411,15 @@ class EntityDescriptor(object):
                         keys = [(self.identity, self.entity.table)]
                         keys.extend([(child._descriptor.identity, child.table)
                                      for child in self._get_children()])
-                        #XXX: we might need to change the alias name so that
-                        # children (which are parent themselves) don't end up
-                        # with the same alias than their parent?
+                        # Having the same alias name for an entity and one of
+                        # its child (which is a parent itself) shouldn't cause
+                        # any problem because the join shouldn't be used at
+                        # the same time. But in reality, some versions of SA
+                        # do misbehave on this. Since it doesn't hurt to have
+                        # different names anyway, here they go.
                         pjoin = polymorphic_union(
-                                    dict(keys), self.polymorphic, 'pjoin')
+                                    dict(keys), self.polymorphic,
+                                    'pjoin_%s' % self.identity)
 
                         kwargs['with_polymorphic'] = ('*', pjoin)
                         kwargs['polymorphic_on'] = \
@@ -376,42 +428,39 @@ class EntityDescriptor(object):
                         kwargs['polymorphic_on'] = \
                             self.get_column(self.polymorphic)
 
-                    #TODO: this is an optimization, and it breaks the multi
-                    # table polymorphic inheritance test with a relation.
-                    # So I turn it off for now. We might want to provide an
-                    # option to turn it on.
-#                    if self.inheritance == 'multi':
-#                        children = self._get_children()
-#                        join = self.entity.table
-#                        for child in children:
-#                            join = join.outerjoin(child.table)
-#                        kwargs['select_table'] = join
-
                 if self.children or self.parent:
                     kwargs['polymorphic_identity'] = self.identity
 
                 if self.parent and self.inheritance == 'concrete':
                     kwargs['concrete'] = True
 
-        if 'primary_key' in kwargs:
-            cols = self.entity.table.c
-            kwargs['primary_key'] = [getattr(cols, colname) for
-                colname in kwargs['primary_key']]
-
         if self.parent and self.inheritance == 'single':
             args = []
         else:
             args = [self.entity.table]
 
+        # let user-defined kwargs override Elixir-generated ones, though that's
+        # not very usefull since most of them expect Column instances.
+        kwargs.update(self.mapper_options)
+
+        #TODO: document this!
+        if 'primary_key' in kwargs:
+            cols = self.entity.table.c
+            kwargs['primary_key'] = [getattr(cols, colname) for
+                colname in kwargs['primary_key']]
+
         # do the mapping
         if self.session is None:
             self.entity.mapper = mapper(self.entity, *args, **kwargs)
         elif isinstance(self.session, ScopedSession):
-            self.entity.mapper = self.session.mapper(self.entity,
-                                                     *args, **kwargs)
+            session_mapper = session_mapper_factory(self.session)
+            self.entity.mapper = session_mapper(self.entity, *args, **kwargs)
         else:
             raise Exception("Failed to map entity '%s' with its table or "
-                            "selectable" % self.entity.__name__)
+                            "selectable. You can only bind an Entity to a "
+                            "ScopedSession object or None for manual session "
+                            "management."
+                            % self.entity.__name__)
 
     def after_mapper(self):
         self.call_builders('after_mapper')
@@ -438,10 +487,13 @@ class EntityDescriptor(object):
         if check_duplicate is None:
             check_duplicate = not self.allowcoloverride
 
-        if check_duplicate and self.get_column(col.key, False) is not None:
-            raise Exception("Column '%s' already exist in '%s' ! " %
-                            (col.key, self.entity.__name__))
-        self._columns.append(col)
+        if col.key in self._columns:
+            if check_duplicate:
+                raise Exception("Column '%s' already exist in '%s' ! " %
+                                (col.key, self.entity.__name__))
+            else:
+                del self._columns[col.key]
+        self._columns.add(col)
 
         if col.primary_key:
             self.has_pk = True
@@ -452,7 +504,7 @@ class EntityDescriptor(object):
         # happen, we'll get an infinite loop. So we just make sure we don't
         # get one in any case.
         table = type.__getattribute__(self.entity, 'table')
-        if table:
+        if table is not None:
             if check_duplicate and col.key in table.columns.keys():
                 raise Exception("Column '%s' already exist in table '%s' ! " %
                                 (col.key, table.name))
@@ -464,7 +516,7 @@ class EntityDescriptor(object):
         self.constraints.append(constraint)
 
         table = self.entity.table
-        if table:
+        if table is not None:
             table.append_constraint(constraint)
 
     def add_property(self, name, property, check_duplicate=True):
@@ -475,7 +527,8 @@ class EntityDescriptor(object):
 
 #FIXME: something like this is needed to propagate the relationships from
 # parent entities to their children in a concrete inheritance scenario. But
-# this doesn't work because of the backref matching code.
+# this doesn't work because of the backref matching code. In most case
+# (test_concrete.py) it doesn't even happen at all.
 #        if self.children and self.inheritance == 'concrete':
 #            for child in self.children:
 #                child._descriptor.add_property(name, property)
@@ -496,13 +549,13 @@ class EntityDescriptor(object):
     def get_column(self, key, check_missing=True):
         #TODO: this needs to work whether the table is already setup or not
         #TODO: support SA table/autoloaded entity
-        for col in self.columns:
-            if col.key == key:
-                return col
-        if check_missing:
-            raise Exception("No column named '%s' found in the table of the "
-                            "'%s' entity!" % (key, self.entity.__name__))
-        return None
+        try:
+            return self.columns[key]
+        except KeyError:
+            if check_missing:
+                raise Exception("No column named '%s' found in the table of "
+                                "the '%s' entity!"
+                                % (key, self.entity.__name__))
 
     def get_inverse_relation(self, rel, check_reverse=True):
         '''
@@ -555,10 +608,7 @@ class EntityDescriptor(object):
     table_fullname = property(table_fullname)
 
     def columns(self):
-        #FIXME: this would be more correct but it breaks inheritance, so I'll
-        # use the old test for now.
-#        if self.entity.table:
-        if self.autoload:
+        if self.entity.table is not None:
             return self.entity.table.columns
         else:
             #FIXME: depending on the type of inheritance, we should also
@@ -582,6 +632,13 @@ class EntityDescriptor(object):
                 return [col for col in self.columns if col.primary_key]
     primary_keys = property(primary_keys)
 
+    def table(self):
+        if self.entity.table is not None:
+            return self.entity.table
+        else:
+            return FakeTable(self)
+    table = property(table)
+
     def primary_key_properties(self):
         """
         Returns the list of (mapper) properties corresponding to the primary
@@ -603,6 +660,36 @@ class EntityDescriptor(object):
         return self._pk_props
     primary_key_properties = property(primary_key_properties)
 
+class FakePK(object):
+    def __init__(self, descriptor):
+        self.descriptor = descriptor
+
+    def columns(self):
+        return self.descriptor.primary_keys
+    columns = property(columns)
+
+class FakeTable(object):
+    def __init__(self, descriptor):
+        self.descriptor = descriptor
+        self.primary_key = FakePK(descriptor)
+
+    def columns(self):
+        return self.descriptor.columns
+    columns = property(columns)
+
+    def fullname(self):
+        '''
+        Complete name of the table for the related entity.
+        Includes the schema name if there is one specified.
+        '''
+        schema = self.descriptor.table_options.get('schema', None)
+        if schema is not None:
+            return "%s.%s" % (schema, self.descriptor.tablename)
+        else:
+            return self.descriptor.tablename
+    fullname = property(fullname)
+
+
 class TriggerProxy(object):
     """
     A class that serves as a "trigger" ; accessing its attributes runs
@@ -652,6 +739,80 @@ def is_entity(cls):
     return False
 
 
+# Note that we don't use inspect.getmembers because of
+# http://bugs.python.org/issue1785
+# See also http://elixir.ematia.de/trac/changeset/262
+def getmembers(object, predicate=None):
+    base_props = []
+    for key in dir(object):
+        try:
+            value = getattr(object, key)
+        except AttributeError:
+            continue
+        if not predicate or predicate(value):
+            base_props.append((key, value))
+    return base_props
+
+def is_abstract_entity(dict_or_cls):
+    if not isinstance(dict_or_cls, dict):
+        dict_or_cls = dict_or_cls.__dict__
+    for mutator, args, kwargs in dict_or_cls.get(MUTATORS, []):
+        if 'abstract' in kwargs:
+            return kwargs['abstract']
+
+    return False
+
+def instrument_class(cls):
+    """
+    Instrument a class as an Entity. This is usually done automatically through
+    the EntityMeta metaclass.
+    """
+    # Create the entity descriptor
+    desc = cls._descriptor = EntityDescriptor(cls)
+
+    # Process mutators
+    # We *do* want mutators to be processed for base/abstract classes
+    # (so that statements like using_options_defaults work).
+    process_mutators(cls)
+
+    # We do not want to do any more processing for base/abstract classes
+    # (Entity et al.).
+    if not is_entity(cls) or is_abstract_entity(cls):
+        return
+
+    cls.table = None
+    cls.mapper = None
+
+    # Copy the properties ('Property' instances) of the entity base class(es).
+    # We use getmembers (instead of __dict__) so that we also get the
+    # properties from the parents of the base class if any.
+    base_props = []
+    for base in cls.__bases__:
+        if isinstance(base, EntityMeta) and \
+           (not is_entity(base) or is_abstract_entity(base)):
+            base_props += [(name, deepcopy(attr)) for name, attr in
+                           getmembers(base, lambda a: isinstance(a, Property))]
+
+    # Process attributes (using the assignment syntax), looking for
+    # 'Property' instances and attaching them to this entity.
+    properties = [(name, attr) for name, attr in cls.__dict__.iteritems()
+                               if isinstance(attr, Property)]
+    sorted_props = sorted(base_props + properties,
+                          key=lambda i: i[1]._counter)
+    for name, prop in sorted_props:
+        prop.attach(cls, name)
+
+    # setup misc options here (like tablename etc.)
+    desc.setup_options()
+
+    # create trigger proxies
+    # TODO: support entity_name... It makes sense only for autoloaded
+    # tables for now, and would make more sense if we support "external"
+    # tables
+    if desc.autosetup:
+        _install_autosetup_triggers(cls)
+
+
 class EntityMeta(type):
     """
     Entity meta class.
@@ -660,56 +821,7 @@ class EntityMeta(type):
     """
 
     def __init__(cls, name, bases, dict_):
-        # Only process further subclasses of the base classes (Entity et al.),
-        # not the base classes themselves. We don't want the base entities to
-        # be registered in an entity collection, nor to have a table name and
-        # so on.
-        if not is_entity(cls):
-            return
-
-        # create the entity descriptor
-        desc = cls._descriptor = EntityDescriptor(cls)
-
-        # Determine whether this entity is a *direct* subclass of its base
-        # entity
-        entity_base = None
-        for base in cls.__bases__:
-            if isinstance(base, EntityMeta):
-                if not is_entity(base):
-                    entity_base = base
-
-        if entity_base:
-            # If so, copy the base entity properties ('Property' instances).
-            # We use inspect.getmembers (instead of __dict__) so that we also
-            # get the properties from the parents of the base_class if any.
-            base_props = inspect.getmembers(entity_base,
-                                            lambda a: isinstance(a, Property))
-            base_props = [(name, copy(attr)) for name, attr in base_props]
-        else:
-            base_props = []
-
-        # Process attributes (using the assignment syntax), looking for
-        # 'Property' instances and attaching them to this entity.
-        properties = [(name, attr) for name, attr in cls.__dict__.iteritems()
-                                   if isinstance(attr, Property)]
-        sorted_props = sorted(base_props + properties,
-                              key=lambda i: i[1]._counter)
-        for name, prop in sorted_props:
-            prop.attach(cls, name)
-
-        # Process mutators. Needed before _install_autosetup_triggers so that
-        # we know of the metadata
-        process_mutators(cls)
-
-        # setup misc options here (like tablename etc.)
-        desc.setup_options()
-
-        # create trigger proxies
-        # TODO: support entity_name... It makes sense only for autoloaded
-        # tables for now, and would make more sense if we support "external"
-        # tables
-        if desc.autosetup:
-            _install_autosetup_triggers(cls)
+        instrument_class(cls)
 
     def __call__(cls, *args, **kwargs):
         if cls._descriptor.autosetup and not hasattr(cls, '_setup_done'):
@@ -730,6 +842,9 @@ class EntityMeta(type):
 def _install_autosetup_triggers(cls, entity_name=None):
     #TODO: move as much as possible of those "_private" values to the
     # descriptor, so that we don't mess the initial class.
+    warnings.warn("The 'autosetup' option on entities is deprecated. "
+        "Please call setup_all() manually after all your entities have been "
+        "declared.", DeprecationWarning, stacklevel=4)
     tablename = cls._descriptor.tablename
     schema = cls._descriptor.table_options.get('schema', None)
     cls._table_key = sqlalchemy.schema._get_table_key(tablename, schema)
@@ -748,16 +863,23 @@ def _install_autosetup_triggers(cls, entity_name=None):
     # see:
     # - table_iterator method in MetaData class in sqlalchemy/schema.py
     # - visit_metadata method in sqlalchemy/ansisql.py
-    original_table_iterator = md.table_iterator
-    if not hasattr(original_table_iterator,
-                   '_non_elixir_patched_iterator'):
-        def table_iterator(*args, **kwargs):
-            elixir.setup_all()
-            return original_table_iterator(*args, **kwargs)
-        table_iterator.__doc__ = original_table_iterator.__doc__
-        table_iterator._non_elixir_patched_iterator = \
-            original_table_iterator
-        md.table_iterator = table_iterator
+    if SA05orlater:
+        warnings.warn(
+            "The automatic setup via metadata.create_all() through "
+            "the autosetup option doesn't work with SQLAlchemy 0.5 and later!")
+    else:
+        # SA 0.6 does not use table_iterator anymore (it was already deprecated
+        # since SA 0.5.0)
+        original_table_iterator = md.table_iterator
+        if not hasattr(original_table_iterator,
+                       '_non_elixir_patched_iterator'):
+            def table_iterator(*args, **kwargs):
+                elixir.setup_all()
+                return original_table_iterator(*args, **kwargs)
+            table_iterator.__doc__ = original_table_iterator.__doc__
+            table_iterator._non_elixir_patched_iterator = \
+                original_table_iterator
+            md.table_iterator = table_iterator
 
     #TODO: we might want to add all columns that will be available as
     #attributes on the class itself (in SA 0.4+). This is a pretty
@@ -787,9 +909,10 @@ def _cleanup_autosetup_triggers(cls):
     md.tables.pop(cls._table_key, None)
 
     # restore original table iterator if not done already
-    if hasattr(md.table_iterator, '_non_elixir_patched_iterator'):
-        md.table_iterator = \
-            md.table_iterator._non_elixir_patched_iterator
+    if not SA05orlater:
+        if hasattr(md.table_iterator, '_non_elixir_patched_iterator'):
+            md.table_iterator = \
+                md.table_iterator._non_elixir_patched_iterator
 
     del cls._has_triggers
 
@@ -798,6 +921,13 @@ def setup_entities(entities):
     '''Setup all entities in the list passed as argument'''
 
     for entity in entities:
+        # delete all Elixir properties so that it doesn't interfere with
+        # SQLAlchemy. At this point they should have be converted to
+        # builders.
+        for name, attr in entity.__dict__.items():
+            if isinstance(attr, Property):
+                delattr(entity, name)
+
         if entity._descriptor.autosetup:
             _cleanup_autosetup_triggers(entity)
 
@@ -808,11 +938,18 @@ def setup_entities(entities):
             'before_mapper', 'setup_mapper', 'after_mapper',
             'setup_properties',
             'finalize'):
+#        if DEBUG:
+#            print "=" * 40
+#            print method_name
+#            print "=" * 40
         for entity in entities:
+#            print entity.__name__, "...",
             if hasattr(entity, '_setup_done'):
+#                print "already done"
                 continue
             method = getattr(entity._descriptor, method_name)
             method()
+#            print "ok"
 
 
 def cleanup_entities(entities):
@@ -842,40 +979,33 @@ def cleanup_entities(entities):
 
         desc._pk_col_done = False
         desc.has_pk = False
-        desc._columns = []
+        desc._columns = ColumnCollection()
         desc.constraints = []
         desc.properties = {}
 
-
-class Entity(object):
-    '''
-    The base class for all entities
-
-    All Elixir model objects should inherit from this class. Statements can
-    appear within the body of the definition of an entity to define its
-    fields, relationships, and other options.
-
-    Here is an example:
+class EntityBase(object):
+    """
+    This class holds all methods of the "Entity" base class, but does not act
+    as a base class itself (it does not use the EntityMeta metaclass), but
+    rather as a parent class for Entity. This is meant so that people who want
+    to provide their own base class but don't want to loose or copy-paste all
+    the methods of Entity can do so by inheriting from EntityBase:
 
     .. sourcecode:: python
 
-        class Person(Entity):
-            name = Field(Unicode(128))
-            birthdate = Field(DateTime, default=datetime.now)
+        class MyBase(EntityBase):
+            __metaclass__ = EntityMeta
 
-    Please note, that if you don't specify any primary keys, Elixir will
-    automatically create one called ``id``.
-
-    For further information, please refer to the provided examples or
-    tutorial.
-    '''
-    __metaclass__ = EntityMeta
+            def myCustomMethod(self):
+                # do something great
+    """
 
     def __init__(self, **kwargs):
-        self.from_dict(kwargs)
+        self.set(**kwargs)
 
     def set(self, **kwargs):
-        self.from_dict(kwargs)
+        for key, value in kwargs.iteritems():
+            setattr(self, key, value)
 
     def update_or_create(cls, data, surrogate=True):
         pk_props = cls._descriptor.primary_key_properties
@@ -949,7 +1079,9 @@ class Entity(object):
             #FIXME: use attribute names (ie coltoprop) instead of column names
             fks = self.mapper.get_property(rname).remote_side
             exclude = [c.name for c in fks]
-            if isinstance(dbdata, list):
+            if dbdata is None:
+                data[rname] = None
+            elif isinstance(dbdata, list):
                 data[rname] = [o.to_dict(rdeep, exclude) for o in dbdata]
             else:
                 data[rname] = dbdata.to_dict(rdeep, exclude)
@@ -995,10 +1127,46 @@ class Entity(object):
 
     # query methods
     def get_by(cls, *args, **kwargs):
+        """
+        Returns the first instance of this class matching the given criteria.
+        This is equivalent to:
+        session.query(MyClass).filter_by(...).first()
+        """
         return cls.query.filter_by(*args, **kwargs).first()
     get_by = classmethod(get_by)
 
     def get(cls, *args, **kwargs):
+        """
+        Return the instance of this class based on the given identifier,
+        or None if not found. This is equivalent to:
+        session.query(MyClass).get(...)
+        """
         return cls.query.get(*args, **kwargs)
     get = classmethod(get)
 
+
+class Entity(EntityBase):
+    '''
+    The base class for all entities
+
+    All Elixir model objects should inherit from this class. Statements can
+    appear within the body of the definition of an entity to define its
+    fields, relationships, and other options.
+
+    Here is an example:
+
+    .. sourcecode:: python
+
+        class Person(Entity):
+            name = Field(Unicode(128))
+            birthdate = Field(DateTime, default=datetime.now)
+
+    Please note, that if you don't specify any primary keys, Elixir will
+    automatically create one called ``id``.
+
+    For further information, please refer to the provided examples or
+    tutorial.
+    '''
+    __metaclass__ = EntityMeta
+
+
