@@ -66,7 +66,7 @@ class EntityBuilder(object):
 
     def create_properties(self):
         '''
-        Subclasses may override this method to add properties to the involved 
+        Subclasses may override this method to add properties to the involved
         entity.
         '''
 
@@ -126,11 +126,6 @@ class Property(EntityBuilder):
         # register this property as a builder
         entity._descriptor.builders.append(self)
 
-        # delete the original attribute so that it doesn't interfere with
-        # SQLAlchemy.
-        if name in entity.__dict__:
-            delattr(entity, name)
-
     def __repr__(self):
         return "Property(%s, %s)" % (self.name, self.entity)
 
@@ -148,12 +143,15 @@ class GenericProperty(Property):
                              (c.quantity * c.unit_price).label('price')))
     '''
 
-    def __init__(self, prop):
-        super(GenericProperty, self).__init__()
+    def __init__(self, prop, *args, **kwargs):
+        super(GenericProperty, self).__init__(*args, **kwargs)
         self.prop = prop
+        #XXX: move this to Property?
+        self.args = args
+        self.kwargs = kwargs
 
     def create_properties(self):
-        if callable(self.prop):
+        if hasattr(self.prop, '__call__'):
             prop_value = self.prop(self.entity.table.c)
         else:
             prop_value = self.prop
@@ -161,6 +159,8 @@ class GenericProperty(Property):
         self.add_mapper_property(self.name, prop_value)
 
     def evaluate_property(self, prop):
+        if self.args or self.kwargs:
+            raise Exception('superfluous arguments passed to GenericProperty')
         return prop
 
 
@@ -169,28 +169,35 @@ class ColumnProperty(GenericProperty):
     A specialized form of the GenericProperty to generate SQLAlchemy
     ``column_property``'s.
 
-    It takes a single argument, which is a function (often
-    given as an anonymous lambda) taking one argument and returning the
-    desired (scalar-returning) SQLAlchemy ClauseElement. That function will be
-    called whenever the entity table is completely defined, and will be given
-    the .c attribute of the entity as argument (as a way to access the entity
-    columns). The ColumnProperty will first wrap your ClauseElement in a label
-    with the same name as the property, then wrap that in a column_property.
+    It takes a function (often given as an anonymous lambda) as its first
+    argument. Other arguments and keyword arguments are forwarded to the
+    column_property construct. That first-argument function must accept exactly
+    one argument and must return the desired (scalar-returning) SQLAlchemy
+    ClauseElement.
+
+    The function will be called whenever the entity table is completely
+    defined, and will be given
+    the .c attribute of the table of the entity as argument (as a way to
+    access the entity columns). The ColumnProperty will first wrap your
+    ClauseElement in an
+    "empty" label (ie it will be labelled automatically during queries),
+    then wrap that in a column_property.
 
     .. sourcecode:: python
 
         class OrderLine(Entity):
             quantity = Field(Float)
             unit_price = Field(Numeric)
-            price = ColumnProperty(lambda c: c.quantity * c.unit_price)
+            price = ColumnProperty(lambda c: c.quantity * c.unit_price,
+                                   deferred=True)
 
     Please look at the `corresponding SQLAlchemy
-    documentation <http://www.sqlalchemy.org/docs/04/mappers.html
-    #advdatamapping_mapper_expressions>`_ for details.
+    documentation <http://www.sqlalchemy.org/docs/05/mappers.html
+    #sql-expressions-as-mapped-attributes>`_ for details.
     '''
 
     def evaluate_property(self, prop):
-        return column_property(prop.label(None))
+        return column_property(prop.label(None), *self.args, **self.kwargs)
 
 
 class Synonym(GenericProperty):
@@ -215,7 +222,7 @@ class Synonym(GenericProperty):
     '''
 
     def evaluate_property(self, prop):
-        return synonym(prop)
+        return synonym(prop, *self.args, **self.kwargs)
 
 #class Composite(GenericProperty):
 #    def __init__(self, prop):
